@@ -1,5 +1,7 @@
 #!/usr/bin/perl -w
 
+# store_dt_on_disk.pl
+
 #use lib '../blib/lib', '../blib/arch';
 
 use strict;
@@ -11,15 +13,17 @@ my $training_datafile = "training.dat";
 
 my $dt = Algorithm::DecisionTree->new( 
                               training_datafile => $training_datafile,
-#                              entropy_threshold => 0.1,
-#                              debug1 => 1                          
-    );
+                              entropy_threshold => 0.1,
+                              max_depth_desired => 3,
+#                             debug1 => 1,                          
+#                             debug2 => 1,
+         );
 
 $dt->get_training_data();
 
 
-#   UNCOMMENT THE FOLLOWING LINE if you would like to see the training
-#   data that was read from the disk file:
+### UNCOMMENT THE NEXT STATEMENT if you would like to see
+### the training data that was read from the disk file:
 #$dt->show_training_data();
 
 
